@@ -7,16 +7,14 @@ package Device::Chip::MPL3115A2;
 
 use strict;
 use warnings;
-use base qw( Device::Chip );
+use base qw( Device::Chip::Base::RegisteredI2C );
 
 use utf8;
 
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 
 use Carp;
 
-use constant PROTOCOL => "I2C";
-
 use Future::Utils qw( repeat );
 
 use Data::Bitfield qw( bitfield boolfield enumfield );
@@ -39,16 +37,13 @@ concepts or features, only the use of this module to access them.
 
 =cut
 
-sub mount
+sub I2C_options
 {
-   my $self = shift;
-   $self->SUPER::mount( @_ )->then( sub {
-      $self->protocol->configure(
-         # This device has a constant address
-         addr        => 0x60,
-         max_bitrate => 400E3,
-      )
-   });
+   return (
+      # This device has a constant address
+      addr        => 0x60,
+      max_bitrate => 100E3,
+   );
 }
 
 use constant WHO_AM_I_ID => 0xC4;
@@ -123,43 +118,18 @@ bitfield CTRL_REG =>
    IPOL2  => boolfield( 20 ),
    PP_OD2 => boolfield( 21 );
 
-sub _mplread
-{
-   my $self = shift;
-   my ( $reg, $len ) = @_;
-
-   $self->protocol->write_then_read( pack( "C", $reg ), $len )
-}
-
-sub _mplwrite
-{
-   my $self = shift;
-   my ( $reg, $val ) = @_;
-
-   $self->protocol->write( pack( "C", $reg ) . $val );
-}
-
-# Raw 8/16-bit integers
-sub _mplread8  { $_[0]->_mplread( $_[1], 1 )
-                  ->then( sub { Future->done( unpack "C", $_[0] ) } ) }
-sub _mplwrite8 { $_[0]->_mplwrite( $_[1], pack "C", $_[2] ) }
-
-sub _mplread16  { $_[0]->_mplread( $_[1], 2 )
-                   ->then( sub { Future->done( unpack "S>", $_[0] ) } ) }
-sub _mplwrite16 { $_[0]->mplwrite( $_[1], pack "S>", $_[2] ) }
-
 # Converted pressure
-sub _mplread_p { $_[0]->_mplread( $_[1], 3 )
+sub _mplread_p { $_[0]->read_reg( $_[1], 3 )
                   ->then( sub { Future->done( unpack( "L>", "\0" . $_[0] ) / 64 ) } ) }
 
 # Converted altitude
-sub _mplread_a { $_[0]->_mplread( $_[1], 3 )
+sub _mplread_a { $_[0]->read_reg( $_[1], 3 )
                   ->then( sub {
                         my ( $msb, $lsb ) = unpack "s>C", $_[0];
                         Future->done( $msb + ( $lsb / 256 ) ); }) }
 
 # Converted temperature
-sub _mplread_t { $_[0]->_mplread( $_[1], 2 )
+sub _mplread_t { $_[0]->read_reg( $_[1], 2 )
                   ->then( sub {
                         my ( $msb, $lsb ) = unpack "cC", $_[0];
                         Future->done( $msb + ( $lsb / 256 ) ) }) }
@@ -171,12 +141,16 @@ L<Future> instances.
 
 =cut
 
-=head2 $config = $mpl->read_config->get
+=head2 read_config
+
+   $config = $mpl->read_config->get
 
 Returns a C<HASH> reference of the contents of control registers C<CTRL_REG1>
 to C<CTRL_REG3>, using fields named from the data sheet.
 
-=head2 $mpl->change_config( %changes )->get
+=head2 change_config
+
+   $mpl->change_config( %changes )->get
 
 Writes updates to the control registers C<CTRL_REG1> to C<CTRL_REG3>. This
 will be performed as a read-modify-write operation, so any fields not given
@@ -194,7 +168,7 @@ sub _cached_read_ctrlreg
 
    defined $self->{configbytes}
       ? return Future->done( $self->{configbytes} )
-      : return $self->_mplread( REG_CTRL_REG1, 3 )
+      : return $self->read_reg( REG_CTRL_REG1, 3 )
 }
 
 sub read_config
@@ -219,13 +193,17 @@ sub change_config
       my $bytes = $self->{configbytes} =
          substr pack( "L<", pack_CTRL_REG( %$config ) ), 0, 3;
 
-      $self->_mplwrite( REG_CTRL_REG1, $bytes );
+      $self->write_reg( REG_CTRL_REG1, $bytes );
    });
 }
 
-=head2 $pressure = $mpl->get_sealevel_pressure->get
+=head2 get_sealevel_pressure
 
-=head2 $mpl->set_sealevel_pressure->get( $pressure )
+=head2 set_sealevel_pressure
+
+   $pressure = $mpl->get_sealevel_pressure->get
+
+   $mpl->set_sealevel_pressure->get( $pressure )
 
 Read or write the barometric pressure calibration register which is used to
 convert pressure to altitude when the chip is in altimeter mode, in Pascals.
@@ -233,12 +211,18 @@ The default value is 101,326 Pa.
 
 =cut
 
-sub get_sealevel_pressure { shift->_mplread16( REG_BAR_IN_MSB )
-                              ->then( sub { Future->done( $_[0] * 2 ) }) }
+sub get_sealevel_pressure {
+   $_[0]->read_reg( REG_BAR_IN_MSB, 2 )
+      ->transform( done => sub { unpack( "S<", $_[0] ) * 2 } );
+}
 
-sub set_sealevel_pressure { $_[0]->_mplwrite16( REG_BAR_IN_MSB, $_[1] / 2 ) }
+sub set_sealevel_pressure {
+   $_[0]->write_reg( REG_BAR_IN_MSB, pack "S<", $_[1] / 2 )
+}
 
-=head2 $pressure = $mpl->read_pressure->get
+=head2 read_pressure
+
+   $pressure = $mpl->read_pressure->get
 
 Returns the value of the C<OUT_P_*> registers, suitably converted into
 Pascals. (The chip must be in barometer mode and must I<not> be in C<RAW> mode
@@ -248,7 +232,9 @@ for the conversion to work).
 
 sub read_pressure { shift->_mplread_p( REG_OUT_P_MSB ) }
 
-=head2 $altitude = $mpl->read_altitude->get
+=head2 read_altitude
+
+   $altitude = $mpl->read_altitude->get
 
 Returns the value of the C<OUT_P_*> registers, suitably converted into metres.
 (The chip must be in altimeter mode and must I<not> be in C<RAW> mode for the
@@ -258,7 +244,9 @@ conversion to work).
 
 sub read_altitude { shift->_mplread_a( REG_OUT_P_MSB ) }
 
-=head2 $temperature = $mpl->read_temperature->get
+=head2 read_temperature
+
+   $temperature = $mpl->read_temperature->get
 
 Returns the value of the C<OUT_T_*> registers, suitable converted into degrees
 C. (The chip must I<not> be in C<RAW> mode for the conversion to work).
@@ -267,16 +255,24 @@ C. (The chip must I<not> be in C<RAW> mode for the conversion to work).
 
 sub read_temperature { shift->_mplread_t( REG_OUT_T_MSB ) }
 
-=head2 $pressure = $mpl->read_min_pressure->get
+=head2 read_min_pressure
 
-=head2 $pressure = $mpl->read_max_pressure->get
+=head2 read_max_pressure
+
+   $pressure = $mpl->read_min_pressure->get
+
+   $pressure = $mpl->read_max_pressure->get
 
 Returns the values of the C<P_MIN> and C<P_MAX> registers, suitably converted
 into Pascals.
 
-=head2 $mpl->clear_min_pressure->get
+=head2 clear_min_pressure
 
-=head2 $mpl->clear_max_pressure->get
+=head2 clear_max_pressure
+
+   $mpl->clear_min_pressure->get
+
+   $mpl->clear_max_pressure->get
 
 Clear the C<P_MIN> or C<P_MAX> registers, resetting them to start again from
 the next measurement.
@@ -286,21 +282,29 @@ the next measurement.
 sub read_min_pressure { shift->_mplread_p( REG_P_MIN_MSB ) }
 sub read_max_pressure { shift->_mplread_p( REG_P_MAX_MSB ) }
 
-sub clear_min_pressure { shift->_mplwrite( REG_P_MIN_MSB, "\x00\x00\x00" ) }
-sub clear_max_pressure { shift->_mplwrite( REG_P_MAX_MSB, "\x00\x00\x00" ) }
+sub clear_min_pressure { shift->write_reg( REG_P_MIN_MSB, "\x00\x00\x00" ) }
+sub clear_max_pressure { shift->write_reg( REG_P_MAX_MSB, "\x00\x00\x00" ) }
 
-=head2 $altitude = $mpl->read_min_altitude->get
+=head2 read_min_altitude
 
-=head2 $altitude = $mpl->read_max_altitude->get
+=head2 read_max_altitude
+
+   $altitude = $mpl->read_min_altitude->get
+
+   $altitude = $mpl->read_max_altitude->get
 
 Returns the values of the C<P_MIN> and C<P_MAX> registers, suitably converted
 into metres.
 
 =cut
 
-=head2 $mpl->clear_min_altitude->get
+=head2 clear_min_altitude
 
-=head2 $mpl->clear_max_altitude->get
+=head2 clear_max_altitude
+
+   $mpl->clear_min_altitude->get
+
+   $mpl->clear_max_altitude->get
 
 Clear the C<P_MIN> or C<P_MAX> registers, resetting them to start again from
 the next measurement.
@@ -313,16 +317,24 @@ sub read_max_altitude { shift->_mplread_a( REG_P_MAX_MSB ) }
 *clear_min_altitude = \&clear_min_pressure;
 *clear_max_altitude = \&clear_max_pressure;
 
-=head2 $temperature = $mpl->read_min_temperature->get
+=head2 read_min_temperature
 
-=head2 $temperature = $mpl->read_max_temperature->get
+=head2 read_max_temperature
+
+   $temperature = $mpl->read_min_temperature->get
+
+   $temperature = $mpl->read_max_temperature->get
 
 Returns the values of the C<T_MIN> and C<T_MAX> registers, suitably converted
 into metres.
 
-=head2 $mpl->clear_min_temperature->get
+=head2 clear_min_temperature
 
-=head2 $mpl->clear_max_temperature->get
+=head2 clear_max_temperature
+
+   $mpl->clear_min_temperature->get
+
+   $mpl->clear_max_temperature->get
 
 Clear the C<T_MIN> or C<T_MAX> registers, resetting them to start again from
 the next measurement.
@@ -332,14 +344,16 @@ the next measurement.
 sub read_min_temperature { shift->_mplread_t( REG_T_MIN_MSB ) }
 sub read_max_temperature { shift->_mplread_t( REG_T_MAX_MSB ) }
 
-sub clear_min_temperature { shift->_mplwrite( REG_T_MIN_MSB, "\x00\x00" ) }
-sub clear_max_temperature { shift->_mplwrite( REG_T_MAX_MSB, "\x00\x00" ) }
+sub clear_min_temperature { shift->write_reg( REG_T_MIN_MSB, "\x00\x00" ) }
+sub clear_max_temperature { shift->write_reg( REG_T_MAX_MSB, "\x00\x00" ) }
 
 =head1 METHODS
 
 =cut
 
-=head2 $mpl->check_id->get
+=head2 check_id
+
+   $mpl->check_id->get
 
 Reads the C<WHO_AM_I> register and checks for a valid ID result. The returned
 future fails if the expected result is not received.
@@ -350,8 +364,9 @@ sub check_id
 {
    my $self = shift;
 
-   $self->_mplread8( REG_WHO_AM_I )->then( sub {
-      my ( $id ) = @_;
+   $self->read_reg( REG_WHO_AM_I, 1 )->then( sub {
+      my ( $val ) = @_;
+      my $id = unpack "C", $val;
       $id == WHO_AM_I_ID or
          die sprintf "Incorrect response from WHO_AM_I register (got %02X, expected %02X)\n",
             $id, WHO_AM_I_ID;
@@ -360,7 +375,9 @@ sub check_id
    });
 }
 
-=head2 $mpl->start_oneshot->get
+=head2 start_oneshot
+
+   $mpl->start_oneshot->get
 
 Sets the C<OST> bit of C<CTRL_REG1> to start a one-shot measurement when in
 standby mode. After calling this method you will need to use
@@ -376,11 +393,13 @@ sub start_oneshot
    $self->_cached_read_ctrlreg->then( sub {
       my ( $bytes ) = @_;
       my $ctrl_reg1 = substr( $bytes, 0, 1 ) | "\x02"; # Set OST bit
-      $self->_mplwrite( REG_CTRL_REG1, $ctrl_reg1 );
+      $self->write_reg( REG_CTRL_REG1, $ctrl_reg1 );
    });
 }
 
-=head2 $mpl->busywait_oneshot->get
+=head2 busywait_oneshot
+
+   $mpl->busywait_oneshot->get
 
 Repeatedly reads the C<OST> bit of C<CTRL_REG1> until it becomes clear.
 
@@ -391,13 +410,15 @@ sub busywait_oneshot
    my $self = shift;
 
    repeat {
-      $self->_mplread( REG_CTRL_REG1, 1 )->then( sub {
+      $self->read_reg( REG_CTRL_REG1, 1 )->then( sub {
          Future->done( ord( $_[0] ) & 0x02 )
       });
    } until => sub { !$_[0]->failure and !$_[0]->get };
 }
 
-=head2 $mpl->oneshot->get
+=head2 oneshot
+
+   $mpl->oneshot->get
 
 A convenient wrapper around C<start_oneshot> and C<busywait_oneshot>.
 
